@@ -2,20 +2,15 @@
 #' @description Accesses \code{api.tradestatistics.io} and
 #' performs different API calls to transform and return tidy data.
 #' @param years Year contained within the years specified in
-#' api.tradestatistics.io/year_range (e.g. \code{c(1980,1985)}, \code{c(1980:1981)} or \code{1980}).
-#' Default set to \code{1962}.
+#' api.tradestatistics.io/year_range (e.g. \code{c(2002,2004)}, \code{c(2002:2004)} or \code{2002}).
+#' Default set to \code{2019}.
 #' @param reporters ISO code for reporter country (e.g. \code{"chl"}, \code{"Chile"} or
 #' \code{c("chl", "Peru")}). Default set to \code{"all"}.
 #' @param partners ISO code for partner country (e.g. \code{"chl"}, \code{"Chile"} or
 #' \code{c("chl", "Peru")}). Default set to \code{"all"}.
-#' @param products HS product codes (e.g. \code{"0101"}, \code{"01"} or search
+#' @param commodities HS commodity codes (e.g. \code{"0101"}, \code{"01"} or search
 #' matches for \code{"apple"})
-#' to filter products. Default set to \code{"all"}.
-#' @param sections unofficial product sections (e.g. \code{"01"} or search
-#' matches for \code{"animals"}) to filter sections Default set to
-#' \code{"all"}.
-#' @param groups HS product groups (e.g. \code{"01"} or search matches for 
-#' \code{"animals"}) to filter groups. Default set to \code{"all"}.
+#' to filter commodities. Default set to \code{"all"}.
 #' @param table Character string to select the table to obtain the data.
 #' Default set to \code{yr} (Year - Reporter).
 #' Run \code{ots_tables} in case of doubt.
@@ -41,25 +36,23 @@
 #' # so these are just shown without evaluation according to CRAN rules
 #'
 #' # Run `ots_countries` to display the full table of countries
-#' # Run `ots_products` to display the full table of products
+#' # Run `ots_commodities` to display the full table of commodities
 #'
-#' # What does Chile export to China? (1980)
-#' ots_create_tidy_data(years = 1980, reporters = "chl", partners = "chn")
+#' # What does Chile export to China? (2002)
+#' ots_create_tidy_data(years = 2002, reporters = "chl", partners = "chn")
 #'
-#' # What can we say about Horses export in Chile and the World? (1980)
-#' ots_create_tidy_data(years = 1980, products = "0101", table = "yc")
-#' ots_create_tidy_data(years = 1980, reporters = "chl", products = "0101", table = "yrc")
+#' # What can we say about Horses export in Chile and the World? (2002)
+#' ots_create_tidy_data(years = 2002, commodities = "010110", table = "yc")
+#' ots_create_tidy_data(years = 2002, reporters = "chl", commodities = "010110", table = "yrc")
 #'
-#' # What can we say about the different types of apples exported by Chile? (1980)
-#' ots_create_tidy_data(years = 1980, reporters = "chl", products = "apple", table = "yrc")
+#' # What can we say about the different types of apples exported by Chile? (2002)
+#' ots_create_tidy_data(years = 2002, reporters = "chl", commodities = "apple", table = "yrc")
 #' }
 #' @keywords functions
-ots_create_tidy_data <- function(years = 2018,
-                                 reporters = "usa",
+ots_create_tidy_data <- function(years = 2019,
+                                 reporters = "all",
                                  partners = "all",
-                                 products = "all",
-                                 sections = "all",
-                                 groups = "all",
+                                 commodities = "all",
                                  table = "yr",
                                  max_attempts = 5,
                                  use_cache = FALSE,
@@ -79,9 +72,7 @@ ots_create_tidy_data <- function(years = 2018,
     years = years,
     reporters = reporters,
     partners = partners,
-    products = products,
-    sections = sections,
-    groups = groups,
+    commodities = commodities,
     table = table,
     max_attempts = max_attempts,
     use_localhost = use_localhost
@@ -94,9 +85,7 @@ ots_create_tidy_data <- function(years = 2018,
 ots_create_tidy_data_unmemoised <- function(years = 2018,
                                             reporters = "usa",
                                             partners = "all",
-                                            products = "all",
-                                            sections = "all",
-                                            groups = "all",
+                                            commodities = "all",
                                             table = "yr",
                                             max_attempts = 5,
                                             use_localhost = FALSE) {
@@ -106,20 +95,16 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
   }
 
   # Check years ----
-  year_depending_queries <- grep("^reporters|rankings$|^y",
+  year_depending_queries <- grep("^reporters|^y|^rtas|^tariffs",
     tradestatistics::ots_tables$table,
     value = T
   )
 
-  url <- "year_range"
-
-  # commented to avoid CRAN problems ----
-  # resp <- HttpClient$new(url = "https://api.tradestatistics.io/")
-  # resp <- resp$get(url)
-  # year_range <- as.vector(unlist(fromJSON(resp$parse(encoding = "UTF-8"))))
-
-  # update this when new data is added ----
-  year_range <- c(1962,2018)
+  if (use_localhost) {
+    year_range <- unlist(fromJSON("http://localhost:8080/year_range"))
+  } else {
+    year_range <- unlist(fromJSON("https://api.tradestatistics.io/year_range"))
+  }
   
   if (all(years %in% min(year_range):max(year_range)) != TRUE &
     table %in% year_depending_queries) {
@@ -127,7 +112,7 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
   }
 
   # Check reporters and partners ----
-  reporter_depending_queries <- grep("^yr",
+  reporter_depending_queries <- grep("^yr|^tariffs",
     tradestatistics::ots_tables$table,
     value = T
   )
@@ -195,124 +180,50 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
     }
   }
 
-  # Check product codes ----
-  product_depending_queries <- grep("c$",
+  # Check commodity codes ----
+  commodities_depending_queries <- grep("c$|c-imputed$|^tariffs",
     tradestatistics::ots_tables$table,
     value = T
   )
 
-  if (!all(as.character(products) %in%
-    tradestatistics::ots_products$product_code) == TRUE &
-    table %in% product_depending_queries) {
+  if (!all(as.character(commodities) %in%
+    tradestatistics::ots_commodities$commodity_code) == TRUE &
+    table %in% commodities_depending_queries) {
 
-    # products without match (wm)
-    products_wm <- products[!products %in%
-      tradestatistics::ots_products$product_code]
+    # commodities without match (wm)
+    commodities_wm <- commodities[!commodities %in%
+      tradestatistics::ots_commodities$commodity_code]
 
-    # product name match (pmm)
+    # commodity name match (pmm)
     pnm <- lapply(
-      seq_along(products_wm),
-      function(x) { tradestatistics::ots_product_code(productname = products_wm[x]) }
+      seq_along(commodities_wm),
+      function(x) { tradestatistics::ots_commodity_code(commodity = commodities_wm[x]) }
     )
     pnm <- rbindlist(pnm)
 
     # group name match (gnm)
     gnm <- lapply(
-      seq_along(products_wm),
-      function(x) { tradestatistics::ots_product_code(productgroup = products_wm[x]) }
+      seq_along(commodities_wm),
+      function(x) { tradestatistics::ots_commodity_code(group = commodities_wm[x]) }
     )
     gnm <- rbindlist(gnm)
 
-    products_wm <- rbind(pnm, gnm, fill = TRUE)
-    products_wm <- unique(products_wm[nchar(product_code) == 4, .(product_code)])
-    products_wm <- as.vector(unlist(products_wm))
+    commodities_wm <- rbind(pnm, gnm, fill = TRUE)
+    commodities_wm <- unique(commodities_wm[nchar(commodity_code) == 4, .(commodity_code)])
+    commodities_wm <- as.vector(unlist(commodities_wm))
 
-    products <- c(products[products %in%
-      tradestatistics::ots_products$product_code], products_wm)
+    commodities <- c(commodities[commodities %in%
+      tradestatistics::ots_commodities$commodity_code], commodities_wm)
     
-    if(length(products) == 0) {
-      products <- NA
+    if(length(commodities) == 0) {
+      commodities <- NA
     }
   }
 
-  if (!all(as.character(products) %in%
-    tradestatistics::ots_products$product_code == TRUE) &
-    table %in% product_depending_queries) {
-    stop("The requested products do not exist. Please check ots_products.")
-  }
-
-  # Check groups ----
-  group_depending_queries <- grep("-ga$",
-                                    tradestatistics::ots_tables$table,
-                                    value = T
-  )
-  
-  unique_groups <- unique(tradestatistics::ots_groups$group_code)
-  unique_groups <- c(unique_groups[!is.na(unique_groups)], "all")
-  
-  if (!all(as.character(groups) %in% unique_groups) == TRUE &
-      table %in% group_depending_queries) {
-    
-    # groups without match (wm)
-    groups_wm <- groups[!groups %in% unique_groups]
-    
-    # group name match (gmm)
-    gnm <- lapply(
-      seq_along(groups_wm),
-      function(x) { tradestatistics::ots_product_code(productgroup = groups_wm[x]) }
-    )
-    gnm <- rbindlist(gnm)
-    
-    groups_wm <- unique(gnm[, .(group_code)])
-    groups_wm <- as.vector(unlist(groups_wm))
-    
-    groups <- c(groups[groups %in% unique_groups], groups_wm)
-    
-    if(length(groups) == 0) {
-      groups <- NA
-    }
-  }
-  
-  if (!all(as.character(groups) %in% unique_groups == TRUE) &
-      table %in% group_depending_queries) {
-    stop("The requested groups do not exist. Please check ots_products.")
-  }
-  
-  # Check sections ----
-  section_depending_queries <- grep("-sa$",
-                                      tradestatistics::ots_tables$table,
-                                      value = T
-  )
-  
-  unique_sections <- tradestatistics::ots_sections_names$section_code
-  unique_sections <- c(unique_sections, "all")
-  
-  if (!all(as.character(sections) %in% unique_sections) == TRUE &
-      table %in% section_depending_queries) {
-    
-    # sections without match (wm)
-    sections_wm <- sections[!sections %in% unique_sections]
-    
-    # sections name match (snm)
-    snm <- lapply(
-      seq_along(sections_wm),
-      function(x) { tradestatistics::ots_product_section(productsection = sections_wm[x]) }
-    )
-    snm <- rbindlist(snm)
-    
-    sections_wm <- unique(snm[, .(section_code)])
-    sections_wm <- as.vector(unlist(sections_wm))
-    
-    sections <- c(sections[sections %in% unique_sections], sections_wm)
-    
-    if(length(sections) == 0) {
-      sections <- NA
-    }
-  }
-  
-  if (!all(as.character(sections) %in% unique_sections == TRUE) &
-      table %in% section_depending_queries) {
-    stop("The requested sections do not exist. Please check ots_products.")
+  if (!all(as.character(commodities) %in%
+    tradestatistics::ots_commodities$commodity_code == TRUE) &
+    table %in% commodities_depending_queries) {
+    stop("The requested commodities do not exist. Please check ots_commodities.")
   }
   
   # Check optional parameters ----
@@ -325,21 +236,11 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
   }
 
   # Read from API ----
-  if (!table %in% product_depending_queries & any(products != "all") == TRUE) {
-    products <- "all"
-    warning("The products argument will be ignored provided that you requested a table without product_code field.")
+  if (!table %in% commodities_depending_queries & any(commodities != "all") == TRUE) {
+    commodities <- "all"
+    warning("The commodities argument will be ignored provided that you requested a table without commodity_code field.")
   }
   
-  if (!table %in% group_depending_queries & any(groups != "all") == TRUE) {
-    groups <- "all"
-    warning("The groups argument will be ignored provided that you requested a table without group_code field.")
-  }
-  
-  if (!table %in% section_depending_queries & any(sections != "all") == TRUE) {
-    sections <- "all"
-    warning("The sections argument will be ignored provided that you requested a table without section_code field.")
-  }
-
   if (is.null(reporters)) {
     reporters <- "all"
     warning("No reporter was specified, therefore all available reporters will be returned.")
@@ -354,9 +255,7 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
     year = years,
     reporter = reporters,
     partner = partners,
-    product = products,
-    group = groups,
-    section = sections,
+    commodity = commodities,
     stringsAsFactors = FALSE
   )
 
@@ -369,9 +268,7 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
         year = condensed_parameters$year[x],
         reporter_iso = condensed_parameters$reporter[x],
         partner_iso = condensed_parameters$partner[x],
-        product_code = condensed_parameters$product[x],
-        group_code = condensed_parameters$group[x],
-        section_code = condensed_parameters$section[x],
+        commodity_code = condensed_parameters$commodity[x],
         use_localhost = use_localhost
       )
     }
@@ -381,8 +278,7 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
   # no data in API message
   if (any("observation" %in% names(data))) {
     warning("The parameters you specified resulted in API calls returning 0 rows.")
-    data <- data[is.na(observation)]
-    data <- data[, observation := NULL]
+    return(data)
   }
 
   # Add attributes based on codes, etc (and join years, if applicable) ------
@@ -411,78 +307,55 @@ ots_create_tidy_data_unmemoised <- function(years = 2018,
                   allow.cartesian = TRUE)
     data <- setnames(data, "country_fullname_english", "partner_fullname_english")
   }
-
-  # include products data
-  if (table %in% product_depending_queries) {
-    data <- tradestatistics::ots_products[data, on = .(product_code), allow.cartesian = TRUE]
-    data <- tradestatistics::ots_products_shortnames[data, on = .(product_code), allow.cartesian = TRUE]
-    data <- tradestatistics::ots_sections[data, on = .(product_code), allow.cartesian = TRUE]
-    
-    data <- tradestatistics::ots_sections_names[data, on = .(section_code), allow.cartesian = TRUE]
-    data <- tradestatistics::ots_sections_shortnames[data, on = .(section_code), allow.cartesian = TRUE]
-    data <- tradestatistics::ots_sections_colors[data, on = .(section_code), allow.cartesian = TRUE]
-
-    data <- data[, `:=`(group_code = substr(product_code, 1, 2))]
-    data <- tradestatistics::ots_groups[data, on = .(group_code), allow.cartesian = TRUE]
-  }
-
-  # include groups data
-  if (table %in% group_depending_queries) {
-    data <- merge(data, tradestatistics::ots_groups,
-          all.x = TRUE, all.y = FALSE,
-          by.x = "top_export_group_code", by.y = "group_code",
-          allow.cartesian = TRUE)
-    data <- setnames(data, "group_fullname_english", "top_export_group_fullname_english")
-    
-    data <- merge(data, tradestatistics::ots_groups,
+  
+  # include commodities data
+  if (table %in% commodities_depending_queries) {
+    data <- merge(data, tradestatistics::ots_commodities,
                   all.x = TRUE, all.y = FALSE,
-                  by.x = "top_import_group_code", by.y = "group_code",
+                  by.x = "commodity_code", by.y = "commodity_code",
                   allow.cartesian = TRUE)
-    data <- setnames(data, "group_fullname_english", "top_import_group_fullname_english")
   }
   
-  if (table == "yc") {
-    data <- merge(data, tradestatistics::ots_countries[, .(country_iso, country_fullname_english)],
-                  all.x = TRUE, all.y = FALSE,
-                  by.x = "top_exporter_iso", by.y = "country_iso",
-                  allow.cartesian = TRUE)
-    data <- setnames(data, "country_fullname_english", "top_exporter_fullname_english")
+  # special YR cases
+  if (grepl("yr-groups", table)) {
+    ots_groups <- unique(tradestatistics::ots_commodities[, c("group_code", "group_fullname_english")])
+    ots_groups <- ots_groups[!is.na(ots_groups$group_code), ]
     
-    data <- merge(data, tradestatistics::ots_countries[, .(country_iso, country_fullname_english)],
-                  all.x = TRUE, all.y = FALSE,
-                  by.x = "top_importer_iso", by.y = "country_iso",
-                  allow.cartesian = TRUE)
-    data <- setnames(data, "country_fullname_english", "top_importer_fullname_english")
+    if (any(colnames(data) %in% "group_code")) {
+      data <- merge(data, ots_groups,
+                    all.x = TRUE, all.y = FALSE,
+                    by.x = "group_code", by.y = "group_code",
+                    allow.cartesian = TRUE)
+    }
+  }
+  
+  if (grepl("yr-sections", table)) {
+    ots_sections <- unique(tradestatistics::ots_commodities[, c("section_code", "section_fullname_english")])
+    ots_sections <- ots_sections[!is.na(ots_sections$section_code), ]
+    
+    if (any(colnames(data) %in% "section_code")) {
+      data <- merge(data, ots_sections,
+                    all.x = TRUE, all.y = FALSE,
+                    by.x = "section_code", by.y = "section_code",
+                    allow.cartesian = TRUE)
+      
+      data$section_fullname_english[is.na(data$section_fullname_english)] <- "Unspecified"
+    }
   }
   
   columns_order <- c("year",
                      grep("^reporter_", colnames(data), value = TRUE),
                      grep("^partner_", colnames(data), value = TRUE),
-                     grep("^section_", colnames(data), value = TRUE),
+                     grep("^commodity_", colnames(data), value = TRUE),
                      grep("^group_", colnames(data), value = TRUE),
-                     grep("^product_", colnames(data), value = TRUE),
-                     grep("^export_", colnames(data), value = TRUE),
-                     grep("^import_", colnames(data), value = TRUE),
-                     grep("^top_export_product_", colnames(data), value = TRUE),
-                     grep("^top_import_product_", colnames(data), value = TRUE),
-                     grep("^top_export_section_", colnames(data), value = TRUE),
-                     grep("^top_export_group_", colnames(data), value = TRUE),
-                     grep("^top_export_trade_", colnames(data), value = TRUE),
-                     grep("^top_import_section_", colnames(data), value = TRUE),
-                     grep("^top_import_group_", colnames(data), value = TRUE),
-                     grep("^top_import_trade_", colnames(data), value = TRUE),
-                     grep("^top_exporter_iso", colnames(data), value = TRUE),
-                     grep("^top_exporter_fullname_", colnames(data), value = TRUE),
-                     grep("^top_exporter_trade_", colnames(data), value = TRUE),
-                     grep("^top_importer_iso", colnames(data), value = TRUE),
-                     grep("^top_importer_fullname_", colnames(data), value = TRUE),
-                     grep("^top_importer_trade_", colnames(data), value = TRUE),
-                     grep("^cci_", colnames(data), value = TRUE),
-                     grep("^pci_", colnames(data), value = TRUE)
+                     grep("^section_", colnames(data), value = TRUE),
+                     grep("^trade_", colnames(data), value = TRUE),
+                     grep("^country|^rta", colnames(data), value = TRUE),
+                     grep("rate|average|line", colnames(data), value = TRUE)
   )
+
   data <- data[, ..columns_order]
-  
-  if (nrow(data) == 0) { warning("The resulting table contains 0 rows.") }
+
   return(data)
 }
 
