@@ -6,10 +6,10 @@
 #' Default set to \code{NULL}.
 #' @param reporter_iso ISO code for reporter country (e.g. \code{"chl"}). Default set to \code{"all"}.
 #' @param partner_iso ISO code for partner country (e.g. \code{"chl"}). Default set to \code{"all"}.
-#' @param product_code HS code (e.g. \code{0101} or \code{01}) to filter products.
+#' @param commodity_code HS code (e.g. \code{0101} or \code{01}) to filter commodities.
 #' Default set to \code{"all"}.
-#' @param table Character string to select the table to obtain the data. Default set to \code{yrpc}
-#' (Year - Reporter - Partner - product).
+#' @param table Character string to select the table to obtain the data. Default set to \code{yr}
+#' (Year - Reporter).
 #' @param max_attempts Number of attempts to retry in case of data retrieving failure.
 #' Default set to \code{5}.
 #' @param use_localhost Logical to determine if the base URL shall be localhost instead
@@ -27,10 +27,10 @@
 #' ots_read_from_api(year = 1980, reporter_iso = "chl", partner_iso = "chn")
 #'
 #' # What can we say about chilean Horses export? (1980)
-#' ots_read_from_api(year = 1980, product_code = "0101", table = "yc")
-#' ots_read_from_api(year = 1980, reporter_iso = "chl", product_code = "0101", table = "yrc")
+#' ots_read_from_api(year = 1980, commodity_code = "0101", table = "yc")
+#' ots_read_from_api(year = 1980, reporter_iso = "chl", commodity_code = "0101", table = "yrc")
 #' ots_read_from_api(
-#'   year = 1980, reporter_iso = "chl", partner_iso = "arg", product_code = "0101",
+#'   year = 1980, reporter_iso = "chl", partner_iso = "arg", commodity_code = "0101",
 #'   table = "yrpc"
 #' )
 #' }
@@ -38,9 +38,7 @@
 ots_read_from_api <- function(year = NULL,
                               reporter_iso = NULL,
                               partner_iso = NULL,
-                              product_code = "all",
-                              group_code = "all",
-                              section_code = "all",
+                              commodity_code = "all",
                               table = "yr",
                               max_attempts = 5,
                               use_localhost = FALSE) {
@@ -49,48 +47,49 @@ ots_read_from_api <- function(year = NULL,
   url <- switch(
     table,
     "countries" = "countries",
-    "products" = "products",
     "reporters" = sprintf("reporters?y=%s", year),
-    "country_rankings" = sprintf("country_rankings?y=%s", year),
-    "product_rankings" = sprintf("product_rankings?y=%s", year),
+    "partners" = sprintf("partners?y=%s", year),
+    "commodities" = "commodities",
     "yrpc" = sprintf(
       "yrpc?y=%s&r=%s&p=%s&c=%s",
-      year, reporter_iso, partner_iso, product_code
+      year, reporter_iso, partner_iso, commodity_code
     ),
-    "yrpc-ga" = sprintf(
-      "yrpc-ga?y=%s&r=%s&p=%s&g=%s",
-      year, reporter_iso, partner_iso, group_code
+    "yrpc-parquet" = sprintf(
+      "yrpc-parquet?y=%s&r=%s&p=%s&c=%s",
+      year, reporter_iso, partner_iso, commodity_code
     ),
-    "yrpc-sa" = sprintf(
-      "yrpc-sa?y=%s&r=%s&p=%s&s=%s",
-      year, reporter_iso, partner_iso, section_code
+    "yrpc-imputed" = sprintf(
+      "yrpc-imputed?y=%s&r=%s&p=%s&c=%s",
+      year, reporter_iso, partner_iso, commodity_code
     ),
-    "yrpc-sga" = sprintf(
-      "yrpc-sga?y=%s&r=%s&p=%s&g=%s&s=%s",
-      year, reporter_iso, partner_iso, group_code, section_code
+    "yrpc-imputed-parquet" = sprintf(
+      "yrpc-imputed-parquet?y=%s&r=%s&p=%s&c=%s",
+      year, reporter_iso, partner_iso, commodity_code
     ),
     "yrp" = sprintf("yrp?y=%s&r=%s&p=%s", year, reporter_iso, partner_iso),
+    "yrp-imputed" = sprintf("yrp-imputed?y=%s&r=%s&p=%s", year, reporter_iso, partner_iso),
     "yrc" = sprintf(
       "yrc?y=%s&r=%s&c=%s",
-      year, reporter_iso, product_code
+      year, reporter_iso, commodity_code
     ),
-    "yrc-ga" = sprintf(
-      "yrc-ga?y=%s&r=%s&c=%s&g=%s",
-      year, reporter_iso, product_code, group_code
-    ),
-    "yrc-sa" = sprintf(
-      "yrc-sa?y=%s&r=%s&c=%s&s=%s",
-      year, reporter_iso, product_code, section_code
-    ),
-    "yrc-sga" = sprintf(
-      "yrc-sga?y=%s&r=%s&c=%s&g=%s&s=%s",
-      year, reporter_iso, product_code, group_code, section_code
+    "yrc-imputed" = sprintf(
+      "yrc-imputed?y=%s&r=%s&c=%s",
+      year, reporter_iso, commodity_code
     ),
     "yr" = sprintf("yr?y=%s&r=%s", year, reporter_iso),
-    "yr-short" = sprintf("yr-short?y=%s&r=%s", year, reporter_iso),
-    "yr-ga" = sprintf("yr-ga?y=%s&r=%s", year, reporter_iso),
-    "yr-sa" = sprintf("yr-sa?y=%s&r=%s", year, reporter_iso),
-    "yc" = sprintf("yc?y=%s&c=%s", year, product_code)
+    "yr-imputed" = sprintf("yr-imputed?y=%s&r=%s", year, reporter_iso),
+    "yr-groups" = sprintf("yr-groups?y=%s&r=%s", year, reporter_iso),
+    "yr-groups-imputed" = sprintf("yr-groups-imputed?y=%s&r=%s", year, reporter_iso),
+    "yr-sections" = sprintf("yr-sections?y=%s&r=%s", year, reporter_iso),
+    "yr-sections-imputed" = sprintf("yr-sections-imputed?y=%s&r=%s", year, reporter_iso),
+    "yc" = sprintf("yc?y=%s&c=%s", year, commodity_code),
+    "yc-imputed" = sprintf("yc-imputed?y=%s&c=%s", year, commodity_code),
+    "years" = "years",
+    "rtas" = sprintf("rtas?y=%s", year),
+    "tariffs" = sprintf(
+      "tariffs?y=%s&r=%s&c=%s",
+      year, reporter_iso, commodity_code
+    )
   )
 
   if (use_localhost == TRUE) {
@@ -106,24 +105,26 @@ ots_read_from_api <- function(year = NULL,
   if (resp$status_code == 200) {
     combination <- paste(year, reporter_iso, partner_iso, sep = ", ")
     
-    if (product_code != "all") {
-      combination <- paste(combination, product_code, sep = ", ")
-    }
-    
-    if (group_code != "all") {
-      combination <- paste(combination, group_code, sep = ", ")
-    }
-    
-    if (section_code != "all") {
-      combination <- paste(combination, section_code, sep = ", ")
+    if (commodity_code != "all") {
+      combination <- paste(combination, commodity_code, sep = ", ")
     }
     
     message(sprintf("Downloading data for the combination %s...", combination))
 
-    data <- try(
-      fromJSON(resp$parse(encoding = "UTF-8"))
-    )
-
+    if (!grepl("parquet", url)) {
+      data <- try(
+        fromJSON(resp$parse(encoding = "UTF-8"))
+      )
+    } else {
+      if (!requireNamespace("arrow", quietly = TRUE)) {
+        stop("`arrow` must be installed for reading parquet data to work")
+      }
+      
+      data <- try(
+        arrow::read_parquet(resp$content)
+      )
+    }
+    
     if (!is.data.frame(data)) {
       stop("It wasn't possible to obtain data. Provided this function tests your internet connection\nyou misspelled a reporter, partner or table, or there was a server problem. Please check and try again.")
     }
@@ -135,8 +136,8 @@ ots_read_from_api <- function(year = NULL,
   } else {
     # otherwise, sleep a second and try again
     Sys.sleep(1)
-    ots_read_from_api(year, reporter_iso, partner_iso, product_code, group_code,
-                      section_code, table, max_attempts = max_attempts - 1,
+    ots_read_from_api(year, reporter_iso, partner_iso, commodity_code, table, 
+                      max_attempts = max_attempts - 1,
                       use_localhost
     )
   }
